@@ -16,31 +16,39 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include "gstjpegdec.h"
 #include "gstjpegenc.h"
+#include "gstsmokeenc.h"
+#include "gstsmokedec.h"
 
 static gboolean
-plugin_init (GstPlugin *plugin)
+plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "jpegenc", GST_RANK_NONE, GST_TYPE_JPEGENC))
+  if (!gst_element_register (plugin, "jpegenc", GST_RANK_NONE,
+          GST_TYPE_JPEGENC))
     return FALSE;
-  
-  if (!gst_element_register (plugin, "jpegdec", GST_RANK_PRIMARY, GST_TYPE_JPEGDEC))
+
+  if (!gst_element_register (plugin, "jpegdec", GST_RANK_PRIMARY,
+          GST_TYPE_JPEGDEC))
     return FALSE;
-  
+
+  if (!gst_element_register (plugin, "smokeenc", GST_RANK_PRIMARY,
+          GST_TYPE_SMOKEENC))
+    return FALSE;
+
+  if (!gst_element_register (plugin, "smokedec", GST_RANK_PRIMARY,
+          GST_TYPE_SMOKEDEC))
+    return FALSE;
+
   return TRUE;
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "jpeg",
-  "JPeg plugin library",
-  plugin_init,
-  VERSION,
-  "LGPL",
-  GST_COPYRIGHT,
-  GST_PACKAGE,
-  GST_ORIGIN)
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "jpeg",
+    "JPeg plugin library",
+    plugin_init, VERSION, "LGPL", GST_PACKAGE, GST_ORIGIN)
