@@ -969,7 +969,6 @@ _find_chain_init (GstOggDemux * ogg)
 static gboolean
 _find_chain_process (GstOggDemux * ogg, ogg_page * page)
 {
-  GstOggPad *pad = gst_ogg_pad_get (ogg, page);
   gint64 start, end;
 
   if (!_find_chain_get_unknown_part (ogg, &start, &end))
@@ -979,7 +978,7 @@ _find_chain_process (GstOggDemux * ogg, ogg_page * page)
      * - we seeked to a point in the known chain
      * - we're now in a part that belongs to the unordered streams
      */
-    g_assert (g_slist_find (ogg->unordered, pad));
+    g_assert (g_slist_find (ogg->unordered, gst_ogg_pad_get (ogg, page)));
     if (!ogg_page_bos (page)) {
       /* broken stream */
       return FALSE;
@@ -1135,9 +1134,9 @@ gst_ogg_demux_iterate (GstFilePad * pad)
         break;
       case 1:
         GST_LOG_OBJECT (ogg,
-            "processing ogg page (serial %d, packet %ld, granule pos %llu",
+            "processing ogg page (serial %d, packet %ld, granule pos %llu, state: %d",
             ogg_page_serialno (&page), ogg_page_pageno (&page),
-            ogg_page_granulepos (&page));
+            ogg_page_granulepos (&page), ogg->state);
         switch (ogg->state) {
           case GST_OGG_STATE_SETUP:
             if (!setup_funcs[ogg->setup_state].process (ogg, &page)) {
@@ -1328,11 +1327,13 @@ gst_ogg_pad_push (GstOggDemux * ogg, GstOggPad * pad)
 
   while (TRUE) {
     ret = ogg_stream_packetout (&pad->stream, &packet);
+    GST_LOG_OBJECT (ogg, "packetout gave %d", ret);
     switch (ret) {
       case 0:
         return;
       case -1:
-        gst_ogg_pad_reset (ogg, pad);
+        /* out of sync, could call gst_ogg_pad_reset() here but ogg can decode
+         * the packet just fine. We should probably send a DISCONT though. */
         break;
       case 1:{
         /* only push data when playing, not during seek or similar */
@@ -1544,11 +1545,17 @@ gst_ogg_demux_plugin_init (GstPlugin * plugin)
 /* prints all info about the element */
 #undef GST_CAT_DEFAULT
 #define GST_CAT_DEFAULT gst_ogg_demux_setup_debug
-#if 0
-/* use a define here so the function name in debugging output stays the same */
+
+#ifdef GST_DISABLE_GST_DEBUG
+
 static void
-gst_ogg_print_pad (GstOggDemux * ogg, GstOggPad * pad)
-#endif
+gst_ogg_print (GstOggDemux * ogg)
+{
+  /* NOP */
+}
+
+#else /* !GST_DISABLE_GST_DEBUG */
+
 #define gst_ogg_print_pad(ogg, _pad) \
 G_STMT_START{\
   GstOggPad *pad = (_pad); \
@@ -1559,7 +1566,8 @@ G_STMT_START{\
       pad->start_offset, pad->start_found ? "" : " (?)", \
       pad->end_offset, pad->end_found ? "" : " (?)"); \
 }G_STMT_END
-     static void gst_ogg_print (GstOggDemux * ogg)
+static void
+gst_ogg_print (GstOggDemux * ogg)
 {
   guint i;
   GSList *walk;
@@ -1580,4 +1588,6 @@ G_STMT_START{\
       gst_ogg_print_pad (ogg, walk->data);
     }
   }
+
 }
+#endif /* GST_DISABLE_GST_DEBUG */
