@@ -1,20 +1,34 @@
-
-
+/* GStreamer
+ * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) <2003> David Schleef <ds@schleef.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 #include <gst/gst.h>
 #include "gsttextoverlay.h"
-/*#include "gsttexttestsrc.h"*/
-/*#include "gstsubparse.h"*/
-/*#include "SDL_blit.h"*/
 
 static GstElementDetails textoverlay_details = {
   "Text Overlay",
   "Filter/Editor/Video",
   "Adds text strings on top of a video buffer",
-  "Gustavo J. A. M. Carneiro <gjc@inescporto.pt>"
+  "David Schleef <ds@schleef.org>"
 };
 
 enum
@@ -639,4 +653,4 @@ plugin_init (GstPlugin * plugin)
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "textoverlay",
-    "Text overlay", plugin_init, VERSION, "GPL", GST_PACKAGE, GST_ORIGIN)
+    "Text overlay", plugin_init, VERSION, "LGPL", GST_PACKAGE, GST_ORIGIN)
