@@ -28,7 +28,7 @@
 /* elementfactory information */
 GstElementDetails gst_speexdec_details = {
   "speex audio decoder",
-  "Codec/Audio/Decoder",
+  "Codec/Decoder/Audio",
   ".speex",
   "Wim Taymans <wim.taymans@chello.be>",
 };
@@ -42,8 +42,8 @@ enum
 
 enum
 {
-  ARG_0,
-  /* FILL ME */
+  ARG_0
+      /* FILL ME */
 };
 
 static void gst_speexdec_base_init (gpointer g_class);
