@@ -473,6 +473,7 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
   vorbisenc->eos = FALSE;
   vorbisenc->header_sent = FALSE;
 
+  vorbisenc->newmediacount = 0;
   vorbisenc->tags = gst_tag_list_new ();
 
   /* we're chained and we can deal with events */
@@ -497,21 +498,24 @@ gst_vorbisenc_get_tag_value (const GstTagList * list, const gchar * tag,
       || (strcmp (tag, GST_TAG_ALBUM_VOLUME_COUNT) == 0)) {
     guint track_no;
 
-    g_assert (gst_tag_list_get_uint_index (list, tag, index, &track_no));
+    if (!gst_tag_list_get_uint_index (list, tag, index, &track_no))
+      g_assert_not_reached ();
     vorbisvalue = g_strdup_printf ("%u", track_no);
   } else if (strcmp (tag, GST_TAG_DATE) == 0) {
     /* FIXME: how are dates represented in vorbis files? */
     GDate *date;
     guint u;
 
-    g_assert (gst_tag_list_get_uint_index (list, tag, index, &u));
+    if (!gst_tag_list_get_uint_index (list, tag, index, &u))
+      g_assert_not_reached ();
     date = g_date_new_julian (u);
     vorbisvalue =
         g_strdup_printf ("%04d-%02d-%02d", (gint) g_date_get_year (date),
         (gint) g_date_get_month (date), (gint) g_date_get_day (date));
     g_date_free (date);
   } else if (gst_tag_get_type (tag) == G_TYPE_STRING) {
-    g_assert (gst_tag_list_get_string_index (list, tag, index, &vorbisvalue));
+    if (!gst_tag_list_get_string_index (list, tag, index, &vorbisvalue))
+      g_assert_not_reached ();
   }
 
   return vorbisvalue;
@@ -805,6 +809,20 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
         }
         gst_pad_event_default (pad, event);
         return;
+      case GST_EVENT_DISCONTINUOUS:
+        if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+	  /* only do for new media events after the first one */
+	  if (vorbisenc->newmediacount++ > 0) {
+	    vorbisenc->setup = FALSE;
+            vorbisenc->header_sent = FALSE;
+            gst_tag_list_free (vorbisenc->tags);
+            vorbisenc->tags = gst_tag_list_new ();
+	    vorbisenc->eos = FALSE;
+	    gst_vorbisenc_setup (vorbisenc);
+          }
+          gst_pad_event_default (pad, event);
+	  break;
+	}
       default:
         gst_pad_event_default (pad, event);
         return;
