@@ -24,6 +24,7 @@
 #endif
 
 #include <string.h>
+#include <gst/gstutils.h>
 
 #include "riff-ids.h"
 #include "riff-read.h"
@@ -149,12 +150,28 @@ gst_riff_read_element_level_up (GstRiffRead * riff)
  * TRUE on success or FALSE on failure.
  */
 
-static gboolean
+gboolean
 gst_riff_peek_head (GstRiffRead * riff,
     guint32 * tag, guint32 * length, guint * level_up)
 {
+  GList *last;
   guint8 *data;
 
+  /* if we're at the end of a chunk, but unaligned, then re-align.
+   * Those are essentially broken files, but unfortunately they
+   * exist. */
+  if ((last = g_list_last (riff->level)) != NULL) {
+    GstRiffLevel *level = last->data;
+    guint64 pos = gst_bytestream_tell (riff->bs);
+
+    if (level->start + level->length - pos < 8) {
+      if (!gst_bytestream_flush (riff->bs, level->start + level->length - pos)) {
+        GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
+        return FALSE;
+      }
+    }
+  }
+
   /* read */
   while (gst_bytestream_peek_bytes (riff->bs, &data, 8) != 8) {
     GstEvent *event = NULL;
@@ -162,9 +179,11 @@ gst_riff_peek_head (GstRiffRead * riff,
 
     /* Here, we might encounter EOS */
     gst_bytestream_get_status (riff->bs, &remaining, &event);
-    if (GST_IS_EVENT (event)) {
+    if (event && GST_IS_EVENT (event)) {
+      gboolean eos = (GST_EVENT_TYPE (event) == GST_EVENT_EOS);
+
       gst_pad_event_default (riff->sinkpad, event);
-      if (GST_EVENT_TYPE (event) == GST_EVENT_EOS)
+      if (eos)
         return FALSE;
     } else {
       GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
@@ -173,9 +192,9 @@ gst_riff_peek_head (GstRiffRead * riff,
   }
 
   /* parse tag + length (if wanted) */
-  *tag = GUINT32_FROM_LE (((guint32 *) data)[0]);
+  *tag = GST_READ_UINT32_LE (data);
   if (length)
-    *length = GUINT32_FROM_LE (((guint32 *) data)[1]);
+    *length = GST_READ_UINT32_LE (((guint32 *) data) + 1);
 
   /* level */
   if (level_up)
@@ -189,16 +208,40 @@ gst_riff_peek_head (GstRiffRead * riff,
  * Return: the data, as a GstBuffer.
  */
 
-static GstBuffer *
-gst_riff_read_element_data (GstRiffRead * riff, guint length)
+GstBuffer *
+gst_riff_read_element_data (GstRiffRead * riff, guint length, guint * got_bytes)
 {
   GstBuffer *buf = NULL;
+  guint32 got;
 
-  if (gst_bytestream_peek (riff->bs, &buf, length) != length) {
-    GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
-    if (buf)
-      gst_buffer_unref (buf);
-    return NULL;
+  while ((got = gst_bytestream_peek (riff->bs, &buf, length)) != length) {
+    /*GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL)); */
+    GstEvent *event = NULL;
+    guint32 remaining;
+
+    gst_bytestream_get_status (riff->bs, &remaining, &event);
+    if (event && GST_IS_EVENT (event)) {
+      gst_pad_event_default (riff->sinkpad, event);
+      if (GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+
+        if (buf)
+          gst_buffer_unref (buf);
+
+        if (got_bytes)
+          *got_bytes = got;
+
+        return NULL;
+      }
+    } else {
+      GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
+      if (buf)
+        gst_buffer_unref (buf);
+
+      if (got_bytes)
+        *got_bytes = got;
+
+      return NULL;
+    }
   }
 
   /* we need 16-bit alignment */
@@ -207,6 +250,9 @@ gst_riff_read_element_data (GstRiffRead * riff, guint length)
 
   gst_bytestream_flush (riff->bs, length);
 
+  if (got_bytes)
+    *got_bytes = got;
+
   return buf;
 }
 
@@ -230,8 +276,8 @@ gst_riff_read_seek (GstRiffRead * riff, guint64 offset)
   gst_bytestream_get_status (riff->bs, &remaining, &event);
   if (event) {
     g_warning ("Unexpected event before seek");
-    gst_event_unref (event);
   }
+
   if (remaining)
     gst_bytestream_flush_fast (riff->bs, remaining);
 
@@ -255,8 +301,10 @@ gst_riff_read_seek (GstRiffRead * riff, guint64 offset)
       GST_WARNING ("No discontinuity event after seek - seek failed");
       break;
     } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
+      GstEventType type = GST_EVENT_TYPE (event);
+
       gst_pad_event_default (riff->sinkpad, event);
-      if (GST_EVENT_TYPE (event) == GST_EVENT_EOS)
+      if (type == GST_EVENT_EOS)
         return NULL;
       event = NULL;
     }
@@ -302,7 +350,7 @@ gst_riff_peek_list (GstRiffRead * riff)
     return 0;
   }
 
-  return GUINT32_FROM_LE (((guint32 *) data)[2]);
+  return GST_READ_UINT32_LE (((guint32 *) data) + 2);
 }
 
 /*
@@ -328,9 +376,13 @@ gst_riff_read_skip (GstRiffRead * riff)
 
   /* see if we have that much data available */
   gst_bytestream_get_status (riff->bs, &remaining, &event);
-  if (event) {
+  if (event && GST_IS_EVENT (event)) {
+    gboolean eos = (GST_EVENT_TYPE (event) == GST_EVENT_EOS);
+
     g_warning ("Unexpected event in skip");
-    gst_event_unref (event);
+    gst_pad_event_default (riff->sinkpad, event);
+    if (eos)
+      return FALSE;
   }
 
   /* yes */
@@ -362,7 +414,7 @@ gst_riff_read_data (GstRiffRead * riff, guint32 * tag, GstBuffer ** buf)
     return FALSE;
   gst_bytestream_flush_fast (riff->bs, 8);
 
-  return ((*buf = gst_riff_read_element_data (riff, length)) != NULL);
+  return ((*buf = gst_riff_read_element_data (riff, length, NULL)) != NULL);
 }
 
 /*
@@ -458,7 +510,8 @@ gst_riff_read_strh (GstRiffRead * riff, gst_riff_strh ** header)
 }
 
 gboolean
-gst_riff_read_strf_vids (GstRiffRead * riff, gst_riff_strf_vids ** header)
+gst_riff_read_strf_vids_with_data (GstRiffRead * riff,
+    gst_riff_strf_vids ** header, GstBuffer ** extradata)
 {
   guint32 tag;
   GstBuffer *buf;
@@ -496,10 +549,18 @@ gst_riff_read_strf_vids (GstRiffRead * riff, gst_riff_strf_vids ** header)
 #endif
 
   /* size checking */
+  *extradata = NULL;
   if (strf->size > GST_BUFFER_SIZE (buf)) {
     g_warning ("strf_vids header gave %d bytes data, only %d available",
         strf->size, GST_BUFFER_SIZE (buf));
     strf->size = GST_BUFFER_SIZE (buf);
+  } else if (strf->size < GST_BUFFER_SIZE (buf)) {
+    gint len;
+
+    len = GST_BUFFER_SIZE (buf) - strf->size - 2;
+    if (len > 0) {
+      *extradata = gst_buffer_create_sub (buf, strf->size + 2, len);
+    }
   }
 
   /* debug */
@@ -516,6 +577,8 @@ gst_riff_read_strf_vids (GstRiffRead * riff, gst_riff_strf_vids ** header)
   GST_INFO (" ypels_meter %d", strf->ypels_meter);
   GST_INFO (" num_colors  %d", strf->num_colors);
   GST_INFO (" imp_colors  %d", strf->imp_colors);
+  if (*extradata)
+    GST_INFO (" %d bytes extra_data", GST_BUFFER_SIZE (*extradata));
 
   gst_buffer_unref (buf);
 
@@ -524,8 +587,25 @@ gst_riff_read_strf_vids (GstRiffRead * riff, gst_riff_strf_vids ** header)
   return TRUE;
 }
 
+/*
+ * Obsolete, use gst_riff_read_strf_vids_with_data ().
+ */
 gboolean
-gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
+gst_riff_read_strf_vids (GstRiffRead * riff, gst_riff_strf_vids ** header)
+{
+  GstBuffer *data = NULL;
+  gboolean ret;
+
+  ret = gst_riff_read_strf_vids_with_data (riff, header, &data);
+  if (data)
+    gst_buffer_unref (data);
+
+  return ret;
+}
+
+gboolean
+gst_riff_read_strf_auds_with_data (GstRiffRead * riff,
+    gst_riff_strf_auds ** header, GstBuffer ** extradata)
 {
   guint32 tag;
   GstBuffer *buf;
@@ -534,7 +614,7 @@ gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
   if (!gst_riff_read_data (riff, &tag, &buf))
     return FALSE;
 
-  if (tag != GST_RIFF_TAG_strf) {
+  if (tag != GST_RIFF_TAG_strf && tag != GST_RIFF_TAG_fmt) {
     g_warning ("Not a strf chunk");
     gst_buffer_unref (buf);
     return FALSE;
@@ -557,6 +637,21 @@ gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
   strf->size = GUINT16_FROM_LE (strf->size);
 #endif
 
+  /* size checking */
+  *extradata = NULL;
+  if (strf->size > GST_BUFFER_SIZE (buf)) {
+    g_warning ("strf_auds header gave %d bytes data, only %d available",
+        strf->size, GST_BUFFER_SIZE (buf));
+    strf->size = GST_BUFFER_SIZE (buf);
+  } else if (strf->size < GST_BUFFER_SIZE (buf)) {
+    gint len;
+
+    len = GST_BUFFER_SIZE (buf) - strf->size - 2;
+    if (len > 0) {
+      *extradata = gst_buffer_create_sub (buf, strf->size + 2, len);
+    }
+  }
+
   /* debug */
   GST_INFO ("strf tag found in context auds:");
   GST_INFO (" format      %d", strf->format);
@@ -565,6 +660,8 @@ gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
   GST_INFO (" av_bps      %d", strf->av_bps);
   GST_INFO (" blockalign  %d", strf->blockalign);
   GST_INFO (" size        %d", strf->size);     /* wordsize, not extrasize! */
+  if (*extradata)
+    GST_INFO (" %d bytes extra_data", GST_BUFFER_SIZE (*extradata));
 
   gst_buffer_unref (buf);
 
@@ -573,6 +670,22 @@ gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
   return TRUE;
 }
 
+/*
+ * Obsolete, use gst_riff_read_strf_auds_with_data ().
+ */
+gboolean
+gst_riff_read_strf_auds (GstRiffRead * riff, gst_riff_strf_auds ** header)
+{
+  GstBuffer *data = NULL;
+  gboolean ret;
+
+  ret = gst_riff_read_strf_auds_with_data (riff, header, &data);
+  if (data)
+    gst_buffer_unref (data);
+
+  return ret;
+}
+
 gboolean
 gst_riff_read_strf_iavs (GstRiffRead * riff, gst_riff_strf_iavs ** header)
 {
@@ -648,7 +761,7 @@ gst_riff_read_list (GstRiffRead * riff, guint32 * tag)
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 4);
-  *tag = GUINT32_FROM_LE (*(guint32 *) data);
+  *tag = GST_READ_UINT32_LE (data);
 
   /* remember level */
   level = g_new (GstRiffLevel, 1);
@@ -777,7 +890,8 @@ gst_riff_read_info (GstRiffRead * riff)
 
       if (name && name[0] != '\0') {
         GValue src = { 0 }
-        , dest = {
+        , dest =
+        {
         0};
         GType dest_type = gst_tag_get_type (type);
 
@@ -810,8 +924,12 @@ gst_riff_read_info (GstRiffRead * riff)
         gst_pad_push (GST_PAD (padlist->data), GST_DATA (event));
       }
     }
+
+    gst_element_found_tags (element, taglist);
+
     gst_event_unref (event);
-    gst_element_found_tags (GST_ELEMENT (riff), taglist);
+  } else {
+    gst_tag_list_free (taglist);
   }
 
   return TRUE;
@@ -843,7 +961,7 @@ gst_riff_read_header (GstRiffRead * riff, guint32 * doctype)
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 4);
-  *doctype = GUINT32_FROM_LE (*(guint32 *) data);
+  *doctype = GST_READ_UINT32_LE (data);
 
   /* remember level */
   level = g_new (GstRiffLevel, 1);
