@@ -132,7 +132,7 @@ enum
 enum
 {
   ARG_0,
-  ARG_AGGRESSIVE,
+  ARG_AGGRESSIVE
 };
 
 #define DEBUG_INIT(bla) \
@@ -149,7 +149,21 @@ GST_STATIC_CAPS ( \
     "rate = (int) [ 1, MAX ], " \
     "channels = (int) [ 1, 2 ], " \
     "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, " \
-    "width = (int) { 8, 16, 32 }, " \
+    "width = (int) 8, " \
+    "depth = (int) [ 1, 8 ], " \
+    "signed = (boolean) { true, false }; " \
+  "audio/x-raw-int, " \
+    "rate = (int) [ 1, MAX ], " \
+    "channels = (int) [ 1, 2 ], " \
+    "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, " \
+    "width = (int) 16, " \
+    "depth = (int) [ 1, 16 ], " \
+    "signed = (boolean) { true, false }; " \
+  "audio/x-raw-int, " \
+    "rate = (int) [ 1, MAX ], " \
+    "channels = (int) [ 1, 2 ], " \
+    "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, " \
+    "width = (int) 32, " \
     "depth = (int) [ 1, 32 ], " \
     "signed = (boolean) { true, false }; " \
   "audio/x-raw-float, " \
@@ -239,11 +253,12 @@ gst_audio_convert_chain (GstPad * pad, GstData * data)
     gst_pad_event_default (pad, GST_EVENT (buf));
     return;
   }
+  g_return_if_fail (GST_IS_BUFFER (buf));
 
   if (!gst_pad_is_negotiated (this->sink)) {
     GST_ELEMENT_ERROR (this, CORE, NEGOTIATION, (NULL),
         ("Sink pad (connected to %s:%s) not negotiated before chain function",
-         GST_DEBUG_PAD_NAME (gst_pad_get_peer (this->sink))));
+            GST_DEBUG_PAD_NAME (gst_pad_get_peer (this->sink))));
     return;
   }
   if (!gst_pad_is_negotiated (this->src)) {
@@ -346,6 +361,10 @@ gst_audio_convert_parse_caps (const GstCaps * gst_caps,
     GST_DEBUG ("could not get some values from structure");
     return FALSE;
   }
+  if (caps->is_int && caps->depth > caps->width) {
+    GST_DEBUG ("width > depth, not allowed - make us advertise correct caps");
+    return FALSE;
+  }
   return TRUE;
 }
 
@@ -366,20 +385,9 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
   this = GST_AUDIO_CONVERT (GST_OBJECT_PARENT (pad));
   otherpad = (pad == this->src ? this->sink : this->src);
 
-  /* negotiate sinkpad first */
-  if (pad == this->src && !gst_pad_is_negotiated (this->sink))
-    return GST_PAD_LINK_DELAYED;
-
   if (!gst_audio_convert_parse_caps (caps, &ac_caps))
     return GST_PAD_LINK_REFUSED;
 
-  /* try setting our caps on the other side first */
-  if (gst_pad_try_set_caps (otherpad, caps) >= GST_PAD_LINK_OK) {
-    this->srccaps = ac_caps;
-    this->sinkcaps = ac_caps;
-    return GST_PAD_LINK_OK;
-  }
-
   /* ok, not those - try setting "any" caps */
   othercaps = gst_pad_get_allowed_caps (otherpad);
   for (i = 0; i < gst_caps_get_size (othercaps); i++) {
@@ -391,7 +399,8 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
         gst_structure_set (structure, "buffer-frames", G_TYPE_INT,
             ac_caps.buffer_frames, NULL);
       } else {
-        gst_structure_set (structure, "buffer-frames", G_TYPE_INT, 0, NULL);
+        gst_structure_set (structure, "buffer-frames", GST_TYPE_INT_RANGE, 0,
+            G_MAXINT, NULL);
       }
     }
   }
@@ -407,10 +416,14 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
     return ret;
 
   /* woohoo, got it */
-  if (!gst_audio_convert_parse_caps (gst_pad_get_negotiated_caps (otherpad),
-          &other_ac_caps)) {
-    g_critical ("internal negotiation error");
-    return GST_PAD_LINK_REFUSED;
+  othercaps = (GstCaps *) gst_pad_get_negotiated_caps (otherpad);
+  if (othercaps) {
+    if (!gst_audio_convert_parse_caps (othercaps, &other_ac_caps)) {
+      g_critical ("internal negotiation error");
+      return GST_PAD_LINK_REFUSED;
+    }
+  } else {
+    other_ac_caps = ac_caps;
   }
 
   if (this->sink == pad) {
@@ -425,35 +438,42 @@ gst_audio_convert_link (GstPad * pad, const GstCaps * caps)
   return GST_PAD_LINK_OK;
 }
 
+/* tries to fixate the given field of the given caps to the given int value */
 gboolean
 _fixate_caps_to_int (GstCaps ** caps, const gchar * field, gint value)
 {
-  GstCaps *try, *intersection;
+  GstCaps *try, *isect_lower, *isect_higher;
   gboolean ret = FALSE;
   guint i;
 
+  /* First try to see if we can fixate by intersecting given caps with
+   * simple audio caps with ranges starting/ending with value */
   try = gst_caps_new_simple ("audio/x-raw-int", field, GST_TYPE_INT_RANGE,
       G_MININT, value - 1, NULL);
   gst_caps_append (try, gst_caps_new_simple ("audio/x-raw-float", field,
           GST_TYPE_INT_RANGE, G_MININT, value - 1, NULL));
-  intersection = gst_caps_intersect (*caps, try);
-  if (!gst_caps_is_empty (intersection)) {
-    gst_caps_free (try);
+  isect_lower = gst_caps_intersect (*caps, try);
+  gst_caps_free (try);
+
+  if (!gst_caps_is_empty (isect_lower)) {
     try = gst_caps_new_simple ("audio/x-raw-int", field, GST_TYPE_INT_RANGE,
         value, G_MAXINT, NULL);
     gst_caps_append (try, gst_caps_new_simple ("audio/x-raw-float", field,
             GST_TYPE_INT_RANGE, value, G_MAXINT, NULL));
-    gst_caps_free (intersection);
-    intersection = gst_caps_intersect (*caps, try);
-    if (!gst_caps_is_empty (intersection)) {
+    isect_higher = gst_caps_intersect (*caps, try);
+    /* FIXME: why choose to end up with the higher range, and not the fixed
+     * value ? */
+    if (!gst_caps_is_empty (isect_higher)) {
       gst_caps_free (*caps);
-      *caps = intersection;
+      *caps = isect_higher;
       ret = TRUE;
     } else {
-      gst_caps_free (intersection);
+      gst_caps_free (isect_higher);
     }
   }
-  gst_caps_free (try);
+  gst_caps_free (isect_lower);
+
+  /* FIXME: why don't we already return here when ret == TRUE ? */
   for (i = 0; i < gst_caps_get_size (*caps); i++) {
     GstStructure *structure = gst_caps_get_structure (*caps, i);
 
@@ -470,21 +490,31 @@ gst_audio_convert_fixate (GstPad * pad, const GstCaps * caps)
   GstAudioConvert *this =
       GST_AUDIO_CONVERT (gst_object_get_parent (GST_OBJECT (pad)));
   GstPad *otherpad = (pad == this->sink ? this->src : this->sink);
-  GstAudioConvertCaps ac_caps =
+  GstAudioConvertCaps try, ac_caps =
       (pad == this->sink ? this->srccaps : this->sinkcaps);
-  GstCaps *copy;
+  GstCaps *copy = gst_caps_copy (caps);
 
-  /* only fixate when we're proxying, so we don't fixate to some crap the other side doesn't want */
-  if (!GST_PAD_IS_NEGOTIATING (otherpad))
-    return NULL;
+  if (!GST_PAD_IS_NEGOTIATING (otherpad)) {
+    try.channels = 2;
+    try.width = 16;
+    try.depth = 16;
+    try.endianness = G_BYTE_ORDER;
+  } else {
+    try.channels = ac_caps.channels;
+    try.width = ac_caps.is_int ? ac_caps.width : 16;
+    try.depth = ac_caps.is_int ? ac_caps.depth : 16;
+    try.endianness = ac_caps.is_int ? ac_caps.endianness : G_BYTE_ORDER;
+  }
 
-  copy = gst_caps_copy (caps);
-  if (_fixate_caps_to_int (&copy, "channels", ac_caps.channels))
+  if (_fixate_caps_to_int (&copy, "channels", try.channels))
     return copy;
-  if (_fixate_caps_to_int (&copy, "width", ac_caps.is_int ? ac_caps.width : 16))
+  if (_fixate_caps_to_int (&copy, "width", try.width))
     return copy;
-  if (_fixate_caps_to_int (&copy, "depth", ac_caps.is_int ? ac_caps.depth : 16))
+  if (_fixate_caps_to_int (&copy, "depth", try.depth))
     return copy;
+  if (_fixate_caps_to_int (&copy, "endianness", try.endianness))
+    return copy;
+
 
   gst_caps_free (copy);
   return NULL;
@@ -518,6 +548,8 @@ gst_audio_convert_get_buffer (GstBuffer * buf, guint size)
 {
   GstBuffer *ret;
 
+  g_assert (GST_IS_BUFFER (buf));
+
   GST_LOG
       ("new buffer of size %u requested. Current is: data: %p - size: %u - maxsize: %u",
       size, buf->data, buf->size, buf->maxsize);
