@@ -5,6 +5,10 @@
  *          Markus Fick <webmaster@mark-f.de> spline + fir-resampler
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "stdafx.h"
 #include "sndfile.h"
 #include <math.h>
