@@ -17,13 +17,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
-/*#define DEBUG_ENABLED */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <gstvideotestsrc.h>
-#include <videotestsrc.h>
+#include "gstvideotestsrc.h"
+#include "videotestsrc.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -31,7 +29,8 @@
 #include <liboil/liboil.h>
 #endif
 
-
+GST_DEBUG_CATEGORY (videotestsrc_debug);
+#define GST_CAT_DEFAULT videotestsrc_debug
 
 /* elementfactory information */
 static GstElementDetails videotestsrc_details =
@@ -56,6 +55,8 @@ enum
       /* FILL ME */
 };
 
+static GstCaps *capslist = NULL;
+
 static void gst_videotestsrc_base_init (gpointer g_class);
 static void gst_videotestsrc_class_init (GstVideotestsrcClass * klass);
 static void gst_videotestsrc_init (GstVideotestsrc * videotestsrc);
@@ -63,7 +64,7 @@ static GstElementStateReturn gst_videotestsrc_change_state (GstElement *
     element);
 static void gst_videotestsrc_set_clock (GstElement * element, GstClock * clock);
 
-static void gst_videotestsrc_set_pattern (GstVideotestsrc * src,
+static void gst_videotestsrc_set_pattern (GstVideotestsrc * videotestsrc,
     int pattern_type);
 static void gst_videotestsrc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -75,6 +76,9 @@ static GstData *gst_videotestsrc_get (GstPad * pad);
 static const GstQueryType *gst_videotestsrc_get_query_types (GstPad * pad);
 static gboolean gst_videotestsrc_src_query (GstPad * pad,
     GstQueryType type, GstFormat * format, gint64 * value);
+static const GstEventMask *gst_videotestsrc_get_event_masks (GstPad * pad);
+static gboolean gst_videotestsrc_handle_src_event (GstPad * pad,
+    GstEvent * event);
 
 static GstElementClass *parent_class = NULL;
 
@@ -126,7 +130,7 @@ gst_videotestsrc_pattern_get_type (void)
   static GEnumValue pattern_types[] = {
     {GST_VIDEOTESTSRC_SMPTE, "smpte", "SMPTE 100% color bars"},
     {GST_VIDEOTESTSRC_SNOW, "snow", "Random (television snow)"},
-    {GST_VIDEOTESTSRC_BLACK, "black", "0% Black"},
+    {GST_VIDEOTESTSRC_BLACK, "black", "100% Black"},
     {0, NULL, NULL},
   };
 
@@ -175,6 +179,9 @@ gst_videotestsrc_class_init (GstVideotestsrcClass * klass)
 
   gstelement_class->change_state = gst_videotestsrc_change_state;
   gstelement_class->set_clock = gst_videotestsrc_set_clock;
+
+  GST_DEBUG_CATEGORY_INIT (videotestsrc_debug, "videotestsrc", 0,
+      "Video Test Source");
 }
 
 static void
@@ -224,14 +231,14 @@ gst_videotestsrc_src_link (GstPad * pad, const GstCaps * caps)
   const GstStructure *structure;
   GstPadLinkReturn ret;
 
-  GST_DEBUG ("gst_videotestsrc_src_link");
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
+  GST_DEBUG_OBJECT (videotestsrc, "linking");
 
   structure = gst_caps_get_structure (caps, 0);
 
   videotestsrc->fourcc = paintinfo_find_by_structure (structure);
   if (!videotestsrc->fourcc) {
-    g_critical ("videotestsrc format not found\n");
+    g_critical ("videotestsrc format not found");
     return GST_PAD_LINK_REFUSED;
   }
 
@@ -244,7 +251,8 @@ gst_videotestsrc_src_link (GstPad * pad, const GstCaps * caps)
 
   videotestsrc->bpp = videotestsrc->fourcc->bitspp;
 
-  GST_DEBUG ("size %d x %d", videotestsrc->width, videotestsrc->height);
+  GST_DEBUG_OBJECT (videotestsrc, "size %dx%d, %f fps", videotestsrc->width,
+      videotestsrc->height, videotestsrc->rate);
 
   return GST_PAD_LINK_OK;
 }
@@ -276,6 +284,7 @@ gst_videotestsrc_change_state (GstElement * element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       videotestsrc->timestamp_offset = 0;
+      videotestsrc->running_time = 0;
       videotestsrc->n_frames = 0;
       break;
     case GST_STATE_READY_TO_NULL:
@@ -288,21 +297,29 @@ gst_videotestsrc_change_state (GstElement * element)
 static GstCaps *
 gst_videotestsrc_get_capslist (void)
 {
-  GstCaps *caps;
-  GstStructure *structure;
-  int i;
+  return gst_caps_copy (capslist);
+}
 
-  caps = gst_caps_new_empty ();
-  for (i = 0; i < n_fourccs; i++) {
-    structure = paint_get_structure (fourcc_list + i);
-    gst_structure_set (structure,
-        "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-        "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-        "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
-    gst_caps_append_structure (caps, structure);
+static void
+generate_capslist (void)
+{
+  if (!capslist) {
+    GstCaps *caps;
+    GstStructure *structure;
+    int i;
+
+    caps = gst_caps_new_empty ();
+    for (i = 0; i < n_fourccs; i++) {
+      structure = paint_get_structure (fourcc_list + i);
+      gst_structure_set (structure,
+          "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+          "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+          "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
+      gst_caps_append_structure (caps, structure);
+    }
+
+    capslist = caps;
   }
-
-  return caps;
 }
 
 #if 0
@@ -339,8 +356,6 @@ gst_videotestsrc_getcaps (GstPad * pad)
 static void
 gst_videotestsrc_init (GstVideotestsrc * videotestsrc)
 {
-  GST_DEBUG ("gst_videotestsrc_init");
-
   videotestsrc->srcpad =
       gst_pad_new_from_template (gst_videotestsrc_src_template_factory (),
       "src");
@@ -355,12 +370,22 @@ gst_videotestsrc_init (GstVideotestsrc * videotestsrc)
   gst_pad_set_query_function (videotestsrc->srcpad, gst_videotestsrc_src_query);
   gst_pad_set_query_type_function (videotestsrc->srcpad,
       gst_videotestsrc_get_query_types);
+  gst_pad_set_event_mask_function (videotestsrc->srcpad,
+      gst_videotestsrc_get_event_masks);
+  gst_pad_set_event_function (videotestsrc->srcpad,
+      gst_videotestsrc_handle_src_event);
+
 
   gst_videotestsrc_set_pattern (videotestsrc, GST_VIDEOTESTSRC_SMPTE);
 
   videotestsrc->num_buffers = -1;
   videotestsrc->num_buffers_left = -1;
   videotestsrc->sync = TRUE;
+  videotestsrc->need_discont = FALSE;
+  videotestsrc->loop = FALSE;
+  videotestsrc->segment_start_frame = -1;
+  videotestsrc->segment_end_frame = -1;
+  videotestsrc->timestamp_offset = 0;
 }
 
 
@@ -386,8 +411,7 @@ gst_videotestsrc_src_query (GstPad * pad,
     case GST_QUERY_POSITION:
       switch (*format) {
         case GST_FORMAT_TIME:
-          *value =
-              videotestsrc->n_frames * GST_SECOND / (double) videotestsrc->rate;
+          *value = videotestsrc->running_time;
           res = TRUE;
           break;
         case GST_FORMAT_DEFAULT:       /* frames */
@@ -405,6 +429,97 @@ gst_videotestsrc_src_query (GstPad * pad,
   return res;
 }
 
+static const GstEventMask *
+gst_videotestsrc_get_event_masks (GstPad * pad)
+{
+  static const GstEventMask src_event_masks[] = {
+    {GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH},
+    {0,}
+  };
+
+  return src_event_masks;
+}
+
+static gboolean
+gst_videotestsrc_handle_src_event (GstPad * pad, GstEvent * event)
+{
+  gboolean res = TRUE;
+  GstVideotestsrc *videotestsrc;
+  gint64 new_n_frames;
+  GstClockTime new_running_time;
+
+  g_return_val_if_fail (pad != NULL, FALSE);
+  g_return_val_if_fail (GST_IS_PAD (pad), FALSE);
+
+  videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
+  new_n_frames = videotestsrc->n_frames;
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK:
+    {
+      /* since we don't do bookkeeping, we just assign a useful meaning
+       * to the time/frame relation after seeking based on current fps */
+      switch (GST_EVENT_SEEK_FORMAT (event)) {
+        case GST_FORMAT_TIME:
+          new_running_time = GST_EVENT_SEEK_OFFSET (event);
+          new_n_frames =
+              GST_EVENT_SEEK_OFFSET (event) * (double) videotestsrc->rate /
+              GST_SECOND;
+          videotestsrc->segment_start_frame = -1;
+          videotestsrc->segment_end_frame = -1;
+          break;
+        case GST_FORMAT_DEFAULT:
+          new_n_frames = GST_EVENT_SEEK_OFFSET (event);
+          new_running_time = GST_EVENT_SEEK_OFFSET (event) /
+              (double) videotestsrc->rate * GST_SECOND;
+          videotestsrc->segment_start_frame = -1;
+          videotestsrc->segment_end_frame = -1;
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    }
+    case GST_EVENT_SEEK_SEGMENT:
+    {
+      switch (GST_EVENT_SEEK_FORMAT (event)) {
+        case GST_FORMAT_TIME:
+          new_n_frames =
+              GST_EVENT_SEEK_OFFSET (event) * (double) videotestsrc->rate /
+              GST_SECOND;
+          videotestsrc->segment_start_frame = new_n_frames;
+          videotestsrc->segment_end_frame =
+              GST_EVENT_SEEK_ENDOFFSET (event) * (double) videotestsrc->rate /
+              GST_SECOND;
+          videotestsrc->loop =
+              GST_EVENT_SEEK_TYPE (event) & GST_SEEK_FLAG_SEGMENT_LOOP;
+          break;
+        case GST_FORMAT_DEFAULT:
+          new_n_frames = GST_EVENT_SEEK_OFFSET (event);
+          videotestsrc->segment_start_frame = new_n_frames;
+          videotestsrc->segment_end_frame = GST_EVENT_SEEK_ENDOFFSET (event);
+          videotestsrc->loop =
+              GST_EVENT_SEEK_TYPE (event) & GST_SEEK_FLAG_SEGMENT_LOOP;
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    }
+    default:
+      res = FALSE;
+      break;
+  }
+
+  if (videotestsrc->n_frames != new_n_frames) {
+    videotestsrc->n_frames = new_n_frames;
+    videotestsrc->need_discont = TRUE;
+  }
+
+  return res;
+}
 
 static GstData *
 gst_videotestsrc_get (GstPad * pad)
@@ -413,12 +528,11 @@ gst_videotestsrc_get (GstPad * pad)
   gulong newsize;
   GstBuffer *buf;
 
-  GST_DEBUG ("gst_videotestsrc_get");
-
   g_return_val_if_fail (pad != NULL, NULL);
   g_return_val_if_fail (GST_IS_PAD (pad), NULL);
 
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
+  GST_LOG_OBJECT (videotestsrc, "get");
 
   if (videotestsrc->fourcc == NULL) {
     GST_ELEMENT_ERROR (videotestsrc, CORE, NEGOTIATION, (NULL),
@@ -426,34 +540,24 @@ gst_videotestsrc_get (GstPad * pad)
     return NULL;
   }
 
-  newsize = gst_videotestsrc_get_size (videotestsrc, videotestsrc->width,
-      videotestsrc->height);
-  g_return_val_if_fail (newsize > 0, NULL);
-
-  GST_DEBUG ("size=%ld %dx%d", newsize, videotestsrc->width,
-      videotestsrc->height);
-
-  buf = gst_pad_alloc_buffer (pad, GST_BUFFER_OFFSET_NONE, newsize);
-  g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
-
-  videotestsrc->make_image (videotestsrc, (void *) GST_BUFFER_DATA (buf),
-      videotestsrc->width, videotestsrc->height);
-
-  if (videotestsrc->sync) {
-    GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
+  if (videotestsrc->need_discont) {
+    GstClockTime ts = videotestsrc->timestamp_offset +
         (videotestsrc->n_frames * GST_SECOND) / (double) videotestsrc->rate;
-    videotestsrc->n_frames++;
 
-    /* FIXME this is not correct if we do QoS */
-    if (videotestsrc->clock) {
-      gst_element_wait (GST_ELEMENT (videotestsrc), GST_BUFFER_TIMESTAMP (buf));
+    videotestsrc->need_discont = FALSE;
+    return GST_DATA (gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, ts,
+            NULL));
+  }
+
+  if ((videotestsrc->segment_end_frame != -1) &&
+      (videotestsrc->n_frames > videotestsrc->segment_end_frame)) {
+    if (videotestsrc->loop) {
+      return GST_DATA (gst_event_new (GST_EVENT_SEGMENT_DONE));
+    } else {
+      gst_element_set_eos (GST_ELEMENT (videotestsrc));
+      return GST_DATA (gst_event_new (GST_EVENT_EOS));
     }
-  } else {
-    GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
-        (videotestsrc->n_frames * GST_SECOND) / (double) videotestsrc->rate;
-    videotestsrc->n_frames++;
   }
-  GST_BUFFER_DURATION (buf) = GST_SECOND / (double) videotestsrc->rate;
 
   if (videotestsrc->num_buffers_left == 0) {
     gst_element_set_eos (GST_ELEMENT (videotestsrc));
@@ -463,24 +567,48 @@ gst_videotestsrc_get (GstPad * pad)
       videotestsrc->num_buffers_left--;
   }
 
+  newsize = gst_videotestsrc_get_size (videotestsrc, videotestsrc->width,
+      videotestsrc->height);
+  g_return_val_if_fail (newsize > 0, NULL);
+
+  GST_LOG_OBJECT (videotestsrc, "creating buffer of %ld bytes for %dx%d image",
+      newsize, videotestsrc->width, videotestsrc->height);
+
+  buf = gst_pad_alloc_buffer (pad, GST_BUFFER_OFFSET_NONE, newsize);
+  g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
+
+  videotestsrc->make_image (videotestsrc, (void *) GST_BUFFER_DATA (buf),
+      videotestsrc->width, videotestsrc->height);
+
+  GST_BUFFER_TIMESTAMP (buf) = videotestsrc->timestamp_offset +
+      videotestsrc->running_time;
+  GST_BUFFER_DURATION (buf) = GST_SECOND / (double) videotestsrc->rate;
+  if (videotestsrc->sync) {
+    /* FIXME this is not correct if we do QoS */
+    if (videotestsrc->clock) {
+      gst_element_wait (GST_ELEMENT (videotestsrc), GST_BUFFER_TIMESTAMP (buf));
+    }
+  }
+  videotestsrc->n_frames++;
+  videotestsrc->running_time += GST_BUFFER_DURATION (buf);
   return GST_DATA (buf);
 }
 
 static void
-gst_videotestsrc_set_pattern (GstVideotestsrc * src, int pattern_type)
+gst_videotestsrc_set_pattern (GstVideotestsrc * videotestsrc, int pattern_type)
 {
-  src->type = pattern_type;
+  videotestsrc->type = pattern_type;
 
-  GST_DEBUG ("setting pattern to %d\n", pattern_type);
+  GST_DEBUG_OBJECT (videotestsrc, "setting pattern to %d", pattern_type);
   switch (pattern_type) {
     case GST_VIDEOTESTSRC_SMPTE:
-      src->make_image = gst_videotestsrc_smpte;
+      videotestsrc->make_image = gst_videotestsrc_smpte;
       break;
     case GST_VIDEOTESTSRC_SNOW:
-      src->make_image = gst_videotestsrc_snow;
+      videotestsrc->make_image = gst_videotestsrc_snow;
       break;
     case GST_VIDEOTESTSRC_BLACK:
-      src->make_image = gst_videotestsrc_black;
+      videotestsrc->make_image = gst_videotestsrc_black;
       break;
     default:
       g_assert_not_reached ();
@@ -491,22 +619,21 @@ static void
 gst_videotestsrc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GstVideotestsrc *src;
+  GstVideotestsrc *videotestsrc;
 
   /* it's not null if we got it, but it might not be ours */
   g_return_if_fail (GST_IS_VIDEOTESTSRC (object));
-  src = GST_VIDEOTESTSRC (object);
+  videotestsrc = GST_VIDEOTESTSRC (object);
 
-  GST_DEBUG ("gst_videotestsrc_set_property");
   switch (prop_id) {
     case ARG_TYPE:
-      gst_videotestsrc_set_pattern (src, g_value_get_enum (value));
+      gst_videotestsrc_set_pattern (videotestsrc, g_value_get_enum (value));
       break;
     case ARG_SYNC:
-      src->sync = g_value_get_boolean (value);
+      videotestsrc->sync = g_value_get_boolean (value);
       break;
     case ARG_NUM_BUFFERS:
-      src->num_buffers = g_value_get_int (value);
+      videotestsrc->num_buffers = g_value_get_int (value);
       break;
     default:
       break;
@@ -517,21 +644,21 @@ static void
 gst_videotestsrc_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
 {
-  GstVideotestsrc *src;
+  GstVideotestsrc *videotestsrc;
 
   /* it's not null if we got it, but it might not be ours */
   g_return_if_fail (GST_IS_VIDEOTESTSRC (object));
-  src = GST_VIDEOTESTSRC (object);
+  videotestsrc = GST_VIDEOTESTSRC (object);
 
   switch (prop_id) {
     case ARG_TYPE:
-      g_value_set_enum (value, src->type);
+      g_value_set_enum (value, videotestsrc->type);
       break;
     case ARG_SYNC:
-      g_value_set_boolean (value, src->sync);
+      g_value_set_boolean (value, videotestsrc->sync);
       break;
     case ARG_NUM_BUFFERS:
-      g_value_set_int (value, src->num_buffers);
+      g_value_set_int (value, videotestsrc->num_buffers);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -547,6 +674,8 @@ plugin_init (GstPlugin * plugin)
   oil_init ();
 #endif
 
+  generate_capslist ();
+
   return gst_element_register (plugin, "videotestsrc", GST_RANK_NONE,
       GST_TYPE_VIDEOTESTSRC);
 }
