@@ -63,7 +63,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-rgb, "
-        "framerate = (double) [ 0.0, MAX ], "
+        "framerate = (double) [ 1.0, 100.0 ], "
         "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
 
@@ -689,7 +689,7 @@ gst_ximagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
 
   /* first calculate the "real" ratio based on the X values;
    * which is the "physical" w/h divided by the w/h in pixels of the display */
-  ratio = xcontext->widthmm * xcontext->height
+  ratio = (gdouble) (xcontext->widthmm * xcontext->height)
       / (xcontext->heightmm * xcontext->width);
 
   /* DirectFB's X in 720x576 reports the physical dimensions wrong, so
@@ -746,7 +746,7 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   if (!xcontext->disp) {
     g_mutex_unlock (ximagesink->x_lock);
     g_free (xcontext);
-    GST_ELEMENT_ERROR (ximagesink, RESOURCE, TOO_LAZY, (NULL),
+    GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
         ("Could not open display"));
     return NULL;
   }
@@ -764,6 +764,9 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
   xcontext->widthmm = DisplayWidthMM (xcontext->disp, xcontext->screen_num);
   xcontext->heightmm = DisplayHeightMM (xcontext->disp, xcontext->screen_num);
 
+  GST_DEBUG_OBJECT (ximagesink, "X reports %dx%d pixels and %d mm x %d mm",
+      xcontext->width, xcontext->height, xcontext->widthmm, xcontext->heightmm);
+
   gst_ximagesink_calculate_pixel_aspect_ratio (xcontext);
 
   /* We get supported pixmap formats at supported depth */
@@ -823,7 +826,7 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
       "blue_mask", G_TYPE_INT, xcontext->visual->blue_mask,
       "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
       "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE, NULL);
+      "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
   if (ximagesink->par) {
     int nom, den;
 
